@@ -23,7 +23,7 @@ weight = ARGV.shift
 source_code_path = ARGV.shift
 out_path = ARGV.shift
 
-if source_code_path.nil? || source_code_path.empty? || !File.exists?(source_code_path)
+if source_code_path.nil? || source_code_path.empty? || !File.exist?(source_code_path)
   puts "Please set the source code path"
   exit
 elsif out_path.nil? || out_path.empty?
@@ -33,7 +33,7 @@ end
 
 source_code = File.read(source_code_path)
 Metasploit::Framework::Compiler::Windows.compile_random_c_to_file(out_path, source_code, weight: weight.to_i)
-if File.exists?(out_path)
+if File.exist?(out_path)
   puts "File saved as #{out_path}"
 end
 rescue SignalException => e
