@@ -95,6 +95,15 @@ class Meterpreter < Rex::Post::Meterpreter::Client
     self.console = Rex::Post::Meterpreter::Ui::Console.new(self)
   end
 
+  def exit
+    begin
+      self.core.shutdown
+    rescue StandardError
+      nil
+    end
+    self.shutdown_passive_dispatcher
+    self.console.stop
+  end
   #
   # Returns the session type as being 'meterpreter'.
   #
@@ -157,7 +166,7 @@ class Meterpreter < Rex::Post::Meterpreter::Client
     # always make sure that the new session has a new guid if it's not already known
     guid = session.session_guid
     if guid == "\x00" * 16
-      guid = [SecureRandom.uuid.gsub(/-/, '')].pack('H*')
+      guid = [SecureRandom.uuid.gsub('-', '')].pack('H*')
       session.core.set_session_guid(guid)
       session.session_guid = guid
       # TODO: New stageless session, do some account in the DB so we can track it later.
@@ -166,7 +175,11 @@ class Meterpreter < Rex::Post::Meterpreter::Client
     end
 
     session.commands.concat(session.core.get_loaded_extension_commands('core'))
-
+    if session.tlv_enc_key[:weak_key?]
+      print_warning("Meterpreter session #{session.sid} is using a weak encryption key.")
+      print_warning('Meterpreter start up operations have been aborted. Use the session at your own risk.')
+      return nil
+    end
     # Unhook the process prior to loading stdapi to reduce logging/inspection by any AV/PSP
     if datastore['AutoUnhookProcess'] == true
       console.run_single('load unhook')
@@ -412,7 +425,11 @@ class Meterpreter < Rex::Post::Meterpreter::Client
 
   def update_session_info
     # sys.config.getuid, and fs.dir.getwd cache their results, so update them
-    fs&.dir&.getwd
+    begin
+      fs&.dir&.getwd
+    rescue Rex::Post::Meterpreter::RequestError => e
+      elog('failed retrieving working directory', error: e)
+    end
     username = self.sys.config.getuid
     sysinfo  = self.sys.config.sysinfo
 
@@ -593,6 +610,10 @@ class Meterpreter < Rex::Post::Meterpreter::Client
     sock
   end
 
+  def supports_udp?
+    true
+  end
+
   #
   # Get a string representation of the current session platform
   #
