@@ -50,6 +50,7 @@ module Exploit::Remote::HttpClient
         OptBool.new('HttpTraceHeadersOnly', [false, 'Show HTTP headers only in HttpTrace', false]),
         OptString.new('HttpTraceColors', [false, 'HTTP request and response colors for HttpTrace (unset to disable)', 'red/blu']),
         OptString.new('SSLServerNameIndication', [ false, 'SSL/TLS Server Name Indication (SNI)', nil]),
+        OptString.new('SSLKeyLogFile', [ false, 'The SSL key log file', ENV['SSLKeyLogFile']]),
       ], self.class
     )
 
@@ -167,7 +168,8 @@ module Exploit::Remote::HttpClient
       client_username,
       client_password,
       comm: opts['comm'],
-      subscriber: http_logger_subscriber
+      subscriber: http_logger_subscriber,
+      sslkeylogfile: sslkeylogfile
     )
 
 
@@ -235,7 +237,8 @@ module Exploit::Remote::HttpClient
   # @raise [Rex::Proto::Http::WebSocket::WebSocketError] raises an exception if the connection fails
   # @return [Rex::Proto::Http::WebSocket::Interface]
   def connect_ws(opts={}, timeout = 20)
-    ws_key = Rex::Text.rand_text_alphanumeric(20)
+    # As per the spec (RFC6455 Section 11.3.1), a Sec-WebSocket-Key is a 16 byte value that has been Base64 encoded.
+    ws_key = Rex::Text.encode_base64(SecureRandom.bytes(16))
     opts['headers'] = opts.fetch('headers', {}).merge({
       'Connection' => 'Upgrade',
       'Upgrade' => 'WebSocket',
@@ -700,6 +703,14 @@ module Exploit::Remote::HttpClient
     datastore['SSLVersion']
   end
 
+  #
+  # Returns the SSL key log file path
+  #
+  # @return [String]
+  def sslkeylogfile
+    datastore['SSLKeyLogFile']
+  end
+
   #
   # Returns the configured proxy list
   #
